//
//
// C++ Implementation for module: Pattern_Miner
//
// Description: 
//
//
// Author: exa
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "Pattern_Miner.hxx"

#include "fp-tree/FP_Tree.hxx"
#include "transaction-db/Transaction_Set.hxx"
//#include "2-items/Count.hxx"
//#include "parallel/Parallel.hxx"

Pattern_Miner::Pattern_Miner(const string& name,
			     Transaction::File_Format format,
			     const string& out, bool dbg)
  : db_name(name), my_ts(name, format), ts(my_ts),
    relative_support(false),
    counter(0), given_counter(false), debug(dbg)
{
  if (out != "") {
    write_output = true;
    pattern_filename = out;
  }
  else write_output = false;
}

Pattern_Miner::Pattern_Miner(Transaction_Set &_ts,
			     const string& out, bool dbg)
  : ts(_ts), relative_support(false),
    counter(0), given_counter(false), debug(dbg)
{
  string fn; ostringstream os(fn);
  os << "patterns";
  db_name = os.str();
}

Pattern_Miner::Pattern_Miner(Transaction_Set &_ts, TS_Counter &ctr,
			     bool dbg)
  : ts(_ts), relative_support(false),
    counter(&ctr), given_counter(true), debug(dbg)
{
  string fn; ostringstream os(fn);
  db_name = os.str();
}

Pattern_Miner::~Pattern_Miner() {
}

void Pattern_Miner::discover_relative(float sup)
{
  real_support = sup;
  relative_support = true;

  discover();
}

void Pattern_Miner::discover_absolute(int sup)
{
  // determine count and frequent items
  support = sup;
  relative_support = false;

  discover();
}

void Pattern_Miner::discover()
{
  map<int,int> pat_counts;	// length -> cardinality map for patterns

  TS_Counter* counter;
  if (!given_counter) {
    counter = new TS_Counter(ts);
  }
  else
    counter = this->counter;

  // determine count and frequent items
  CPU_Time t_s;
  if (!given_counter)
    counter->count_items_absolute(support);

  if (debug) {
    cout << endl << "F = " << counter->F
	 << " count = " << counter->count << endl;
  }
  CPU_Time t_f;

  // construct fp tree
  FP_Tree fp_tree(counter->F, counter->count, pat_counts, debug);
  Transaction_Set::Scanner scanner(ts);
  while (!scanner.end()) {
    Transaction t_i;
    if (scanner.read(t_i)) {
      fp_tree.insert(t_i);
    }
  }
  CPU_Time t_d;

  // discover patterns
  if (write_output) {
    ofstream pattern_out(pattern_filename.c_str());
    pattern_out << "(" << counter->num_transactions << ")" << endl;
    fp_tree.discover_patterns(support, &pattern_out);
  }
  else {
    fp_tree.discover_patterns(support);
  }
  CPU_Time t_e;
  if (write_output)
    fp_tree.print_pattern_counts(cout);

  if (!given_counter)
    delete counter;
}
